/* -*-c++-*- Producer - Copyright (C) 2001-2006  Don Burns
 *
 * This library is open source and may be redistributed and/or modified under
 * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
 */
#ifndef FLKT_RENDER_SURFACE_DEF
#define FLKT_RENDER_SURFACE_DEF 1

#include <Producer/RenderSurface>
#include <FL/Fl_Gl_Window.H>

class FLTKRenderSurface: public Producer::RenderSurface, public Fl_Gl_Window
{
    public:

        FLTKRenderSurface( int X, int Y, int W, int H, const char *L="" ):
            Fl_Gl_Window(X, Y, W, H, L),
            _mx(0),
            _my(0),
            _mbutton(0)

        {
        }

        virtual bool realize( Producer::VisualChooser *vc=NULL, Producer::GLContext sharedGLContext=0 ) 
        {
            return _realized = true;
        }

        virtual void getWindowRectangle( int &x, int &y, unsigned int &width, unsigned int &height ) const
        {
            x = this->x();
            y = this->y();
            width = this->w();
            height = this->h();
        }

        virtual bool makeCurrent( bool ) 
        {
            make_current();
            return true;
        }

        virtual void swapBuffers()
        {
            swap_buffers();
        }

        virtual void draw()
        {
            if( _drawCallback != NULL )
                (*_drawCallback)(this, _drawCallbackData );
        }

        virtual void setDrawCallback( Fl_Callback *callback, void *data )
        {
            _drawCallback = callback;
            _drawCallbackData = data;
        }

        int handle(int event)
        {
            switch(event)
            {
                case FL_PUSH : 
                    _mx = Fl::event_x();
                    _my = Fl::event_y();
                    _mbutton |= 1 << (Fl::event_button()-1);
                    redraw();
                    return 1;

                case FL_RELEASE:
                    _mx = Fl::event_x();
                    _my = Fl::event_y();
                    _mbutton &= ~(1 << (Fl::event_button()-1));
                    redraw();
                    return 1;

                case FL_DRAG :
                    _mx = Fl::event_x();
                    _my = Fl::event_y();
                    redraw();
                    return 1;


                case FL_FOCUS :
                    redraw();
                    return 1;

                default:
                // pass other events to the base class...
                return Fl_Gl_Window::handle(event);
            }
        }


    private:
            Fl_Callback *_drawCallback;
            void *_drawCallbackData;

    protected:
            // Button State
            int _mx, _my;
            unsigned int _mbutton;
};

#endif

