/* -*-c++-*- Producer - Copyright (C) 2001-2006  Don Burns
 *
 * This library is open source and may be redistributed and/or modified under
 * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
 */
#include <iostream>

#include "glut.h"

#include <osgDB/ReadFile>
#include <Producer/Camera>
#include <Producer/Trackball>
#include <ProducerOSG/OsgSceneHandler.h>

#include "GLUTRenderSurface.h"

struct AppInfo 
{
    Producer::ref_ptr<GLUTRenderSurface> glutRenderSurface;
    Producer::ref_ptr<Producer::Camera> camera;
    Producer::ref_ptr<Producer::Trackball> trackball;
    float nx, ny;
    unsigned int mbutton;

    AppInfo(): nx(0.0), ny(0.0), mbutton(0) {}

    void normalizeMouseXY( int mx, int my )
    {
        int wx, wy;
        unsigned int ww, wh;
        glutRenderSurface->getWindowRectangle( wx, wy, ww, wh );

        nx = -1.0 + (2 * (float)mx/(float)ww);
        ny =  1.0 - (2 * (float)my/(float)wh);
    }

}appInfo;

static void frame()
{
    // Update the manipulator
    appInfo.trackball->input(appInfo.nx,appInfo.ny,appInfo.mbutton);
    // Set the camera's view
    appInfo.camera->setViewByMatrix( appInfo.trackball->getMatrix() );
    // Render the frame
    appInfo.camera->frame();
}

void mouseButtonCallback( int button, int state, int x, int y )
{
    if( state == GLUT_DOWN )
        appInfo.mbutton |= (1<<button);
    else
        appInfo.mbutton &= ~(1<<button);

    appInfo.normalizeMouseXY( x, y );
}

void mouseMotionCallback( int x, int y )
{
    appInfo.normalizeMouseXY( x, y );
}

void keyboardCallback( unsigned char key, int, int )
{
    switch( key )
    {
        case ' ':
            appInfo.trackball->reset();
            break;
    }
}

int main( int argc, char **argv )
{
    osg::ref_ptr<osg::Node> scene = osgDB::readNodeFile( "cow.osg" );
    if( !scene.valid() )
    {
        std::cerr << "Can't find the cow!" << std::endl;
        return 1;
    }

    glutInit(&argc, argv);

    appInfo.camera = new Producer::Camera;
    // Set up the RenderSurface
    appInfo.glutRenderSurface = new GLUTRenderSurface( 10, 10, 640, 480);
    appInfo.camera->setRenderSurface( appInfo.glutRenderSurface.get() );

    // Set up the scene handler
    Producer::ref_ptr<Producer::OsgSceneHandler> osgSceneHandler = new Producer::OsgSceneHandler;
    osgSceneHandler->setScene( scene.get() );
    appInfo.camera->setSceneHandler( osgSceneHandler.get() );

    // Initialize the Trackball
    Producer::Vec3 sceneCenter;
    float sceneRadius;
    osgSceneHandler->getSceneBounds( sceneCenter, sceneRadius );

    appInfo.trackball = new Producer::Trackball;
    appInfo.trackball->setOrientation( Producer::Trackball::Z_UP );
    appInfo.trackball->translate(-sceneCenter[0],-sceneCenter[1],-sceneCenter[2]);
    appInfo.trackball->setDistance(sceneRadius*3.0f);
    appInfo.trackball->setReference();

    // Set GLUT callbacks
    glutMouseFunc( mouseButtonCallback );
    glutMotionFunc( mouseMotionCallback );
    glutKeyboardFunc( keyboardCallback );

    // We'll update with an idle callback
    glutIdleFunc( &frame );

    glutMainLoop();
    return 0;
}

