/* -*-c++-*- Producer - Copyright (C) 2001-2006  Don Burns
 *
 * This library is open source and may be redistributed and/or modified under
 * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
 */

#include <Producer/Camera>
#include <Producer/Trackball>

#include "OpenGLSceneHandler.h"
#include "KeyboardMouseResponder.h"

int main( int argc, char **argv )
{
    // RenderSurface
    // -------------
    // Create the Render Surface - In this case a Window at x,y = 100,100 width,height=640,480
    //Producer::ref_ptr<Producer::RenderSurface> renderSurface = new Producer::RenderSurface;
    //renderSurface->setWindowRectangle( 100, 100, 640, 480 );
    //renderSurface->setWindowName( "Simple Producer Based OpenGL Viewer Example" );

    // Create a Camera to View the Scene and provide it with the RenderSurface
    Producer::ref_ptr<Producer::Camera> camera = new Producer::Camera;
    //camera->setRenderSurface( renderSurface.get() );

    // OpenGL Scene Handler
    Producer::ref_ptr<Producer::Camera::SceneHandler> sceneHandler = new OpenGLSceneHandler;
    camera->setSceneHandler( sceneHandler.get() );

    // Find out the bounds of the scene
    Producer::Vec3 sceneCenter;
    double sceneRadius;
    sceneHandler->getSceneBounds( sceneCenter, sceneRadius );

    // InputDevice
    // -----------
    // Create a KeyboardMouse
    Producer::ref_ptr<Producer::KeyboardMouse> keyboardMouse = new Producer::KeyboardMouse( camera->getRenderSurface() );

    // Tell it what its callback is, and kick off a background thread to do 
    // Keybard amd Mouse processing.  The done flag is passed by reference 
    // and will be set when the ESC key is pressed
    bool done = false;
    Producer::ref_ptr<KeyboardMouseResponder> kbmcb = new KeyboardMouseResponder(done);
    keyboardMouse->addResponder( kbmcb.get() );
    keyboardMouse->startThread();

    // Manipulator
    // -----------
    // Create the trackball manipulator and ...
    Producer::ref_ptr<Producer::Trackball> trackball = new Producer::Trackball;

    //      ...enable a 'Z' up orientation,
    trackball->setOrientation( Producer::Trackball::Y_UP );

    //      ...Center the camera on the scene,
    trackball->translate(-sceneCenter[0],-sceneCenter[1],-sceneCenter[2]);

    //      ...Move it back the size of the scene's radius times three,
    trackball->setDistance(sceneRadius*3.0f);

    //      ... Set the reference point.  This is the position where the trackball 
    //         will return to when we call trackball->reset();
    trackball->setReference();

    // 'done' will be set by our Keyboard and Mouse callback when we press the 'ESC' key.
    while( !done )
    {
        // Call update on the scene
        sceneHandler->update();

        // Toggle backface culling on an 'c' key press
        if( kbmcb->toggleBackfaceCulling() )
            sceneHandler->toggleBackfaceCulling();

        // Toggle lighing on an 'l' key press
        if( kbmcb->toggleLighting() )
            sceneHandler->toggleLighting();

        // Toggle texturing on an 't' key press
        if( kbmcb->toggleTexturing() )
            sceneHandler->toggleTexturing();

        // Toggle wireframe on an 'w' key press
        if( kbmcb->toggleWireframe() )
            sceneHandler->toggleWireframe();

        // Reset the Trackball if the user presses the <space bar>
        if( kbmcb->resetTrackball() )
            trackball->reset();

        // Update the view, by updating the Trackball with the most recent
        // mouse X, Y, and button state 
        trackball->input( kbmcb->mx(), kbmcb->my(), kbmcb->mbutton() );

        // Camera View Control
        // ------------
        // Set the camera's view by the trackball's matrix
        camera->setViewByMatrix( trackball->getMatrix() );

        // Call the next cull/dispatch frame
        camera->frame();
    }
    return 0;
}
