/* -*-c++-*- Producer - Copyright (C) 2001-2006  Don Burns
 *
 * This library is open source and may be redistributed and/or modified under
 * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
 */

#ifndef PRODUCER_EXAMPLE_KEYBOARD_MOUSE_CALLBACK_DEF
#define PRODUCER_EXAMPLE_KEYBOARD_MOUSE_CALLBACK_DEF 1

#include <Producer/KeyboardMouse>

class KeyboardMouseCallback : public Producer::KeyboardMouseCallback
{
    public:
		KeyboardMouseCallback(bool &done) :
			Producer::KeyboardMouseCallback(),
			_mx(0.0f),_my(0.0f),_mbutton(0),
			_done(done),
            _resetTrackball(false),
            _toggleLighting(false),
            _toggleTexturing(false),
            _toggleWireframe(false),
            _toggleBackfaceCulling(false)
			{}

		virtual void specialKeyPress( Producer::KeyCharacter key )
		{
			switch( key )
			{
				case Producer::KeyChar_Escape:
					_done = true;
					break;

                default:
                    break;
			}
		}

        virtual void shutdown()
        {
            _done = true; 
        }

		virtual void keyPress( Producer::KeyCharacter key )
		{
            switch( key )
            {
                case Producer::KeyChar_b:
                    _toggleBackfaceCulling = true;
                    break;

                case Producer::KeyChar_l:
                    _toggleLighting = true;
                    break;

                case Producer::KeyChar_t:
                    _toggleTexturing = true;
                    break;

                case Producer::KeyChar_w:
                    _toggleWireframe = true;
                    break;

                case Producer::KeyChar_space:
                    _resetTrackball = true;
                    break;

                default:
                    break;
            }
		}

		virtual void mouseMotion( float mx, float my ) 
		{
			_mx = mx;
			_my = my;
		}
		virtual void buttonPress( float mx, float my, unsigned int mbutton ) 
		{
			_mx = mx;
			_my = my;
			_mbutton |= (1<<(mbutton-1));
		}
		virtual void buttonRelease( float mx, float my, unsigned int mbutton ) 
		{
			_mx = mx;
			_my = my;
			_mbutton &= ~(1<<(mbutton-1));
		}

		bool done() { return _done; }
		float mx()  { return _mx; }
		float my()  { return _my; }
		unsigned int mbutton()  { return _mbutton; }

        bool resetTrackball() { return _checkAndResetToggle( _resetTrackball ); }
        bool toggleLighting() { return _checkAndResetToggle( _toggleLighting ); }
        bool toggleTexturing() { return _checkAndResetToggle( _toggleTexturing ); }
        bool toggleWireframe() { return _checkAndResetToggle( _toggleWireframe ); }
        bool toggleBackfaceCulling() { return _checkAndResetToggle( _toggleBackfaceCulling ); }

    protected:
		virtual ~KeyboardMouseCallback() {}

    private:
		float _mx, _my;
		unsigned int _mbutton;
		bool &_done;
        bool _resetTrackball;
        bool _toggleLighting;
        bool _toggleTexturing;
        bool _toggleWireframe;
        bool _toggleBackfaceCulling;

        bool _checkAndResetToggle( bool &toggle )
        {
            return toggle == true ? !(toggle=false) : false;
        }

};

#endif
